==> gatsby-theme-backend-1.0.15.tgz: package/src/components/Backend/Settings/Languages.tsx <==
import React, { useState, useEffect } from "react"
import { isAdministrator } from "../../../services/auth"
import { decodeUnicode } from "../../../utils/helper"
import { navigate } from "gatsby"
import type { ProColumns } from "@ant-design/pro-table"
import { EditableProTable } from "@ant-design/pro-table"
import { Button, notification } from "antd"
import jsonLanguages from "../../../settings/languages.json"
import "./Languages.less"

type DataSourceType = {
	key?: string
	title?: string
	path?: string
	locale?: string
	message?: string
	enabled?: boolean
	default?: boolean
}

const defaultData: DataSourceType[] = jsonLanguages

export default () => {
	const [editableKeys, setEditableRowKeys] = useState<React.Key[]>(() =>
		defaultData.map((item) => item.key)
	)
	const [dataSource, setDataSource] = useState<DataSourceType[]>([])
	const [sha, setSha] = useState("")

	const columns: ProColumns<DataSourceType>[] = [
		{
			title: "Title",
			dataIndex: "title",
			editable: false,
		},
		{
			title: "Path",
			dataIndex: "path",
			editable: false,
		},
		{
			title: "Enabled",
			dataIndex: "enabled",
			valueType: "switch",
		},
		{
			title: "Default",
			dataIndex: "default",
			valueType: "switch",
		},
	]

	const handleGitGet = async () => {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
					duration: 0,
				})
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	const handleGitUpdate = async (contentStr: any) => {
		try {
			const jsonStr = JSON.stringify(contentStr)
			const data = {
				message:
					"[skip ci] Update @maad-dev/gatsby-theme-backend/settings/languages.json from Modern backend.",
				content: btoa(jsonStr),
				sha: sha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error fetchGitApi PUT ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			} else {
				notification["info"]({
					message: "Empty Fetch",
					duration: 0,
				})
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}

		const fetchDataSource = async () => {
			return await handleGitGet().then((getFile) => {
				if (getFile !== false) {
					const decodeContent = decodeUnicode(getFile.content)
					setDataSource(JSON.parse(decodeContent))
					setSha(getFile.sha)
				}
			})
		}
		fetchDataSource()
	}, [])

	return (
		<EditableProTable<DataSourceType>
			columns={columns}
			rowKey="key"
			value={dataSource}
			onChange={setDataSource}
			recordCreatorProps={false}
			toolBarRender={() => {
				return [
					<Button
						type="primary"
						key="save"
						onClick={async () => {
							console.log("save async")
							handleGitUpdate(dataSource).then((updateFile: any) => {
								if (updateFile) {
									notification["success"]({
										message: "Success!",
										description: "Updated languages successfully.",
										duration: 0,
									})
								} else {
									let errMsg = "No status"
									if (updateFile === 404) {
										errMsg = "Status: 404 Not Found"
									} else if (updateFile === 409) {
										errMsg = "Status: 409 Conflict"
									} else if (updateFile === 422) {
										errMsg = "Status: 422 Unprocessable Entity"
									}
									notification["error"]({
										message: "Error!",
										description: errMsg,
										duration: 0,
									})
								}
							})
						}}
					>
						Save
					</Button>,
				]
			}}
			editable={{
				type: "multiple",
				editableKeys,
				onValuesChange: (record, recordList) => {
					setDataSource(recordList)
				},
				onChange: setEditableRowKeys,
			}}
		/>
	)
}

==> gatsby-theme-backend-1.0.16.tgz: package/src/components/Backend/Settings/Languages.tsx <==
import React, { useState, useEffect } from "react"
import { isAdministrator } from "../../../services/auth"
import { decodeUnicode } from "../../../utils/helper"
import { navigate } from "gatsby"
import type { ProColumns } from "@ant-design/pro-table"
import { EditableProTable } from "@ant-design/pro-table"
import { Button, notification } from "antd"
import jsonLanguages from "../../../settings/languages.json"
import "./Languages.less"

type DataSourceType = {
	key?: string
	title?: string
	path?: string
	locale?: string
	message?: string
	enabled?: boolean
	default?: boolean
}

const defaultData: DataSourceType[] = jsonLanguages

export default () => {
	const [editableKeys, setEditableRowKeys] = useState<React.Key[]>(() =>
		defaultData.map((item) => item.key)
	)
	const [dataSource, setDataSource] = useState<DataSourceType[]>([])
	const [sha, setSha] = useState("")

	const columns: ProColumns<DataSourceType>[] = [
		{
			title: "Title",
			dataIndex: "title",
			editable: false,
		},
		{
			title: "Path",
			dataIndex: "path",
			editable: false,
		},
		{
			title: "Enabled",
			dataIndex: "enabled",
			valueType: "switch",
		},
		{
			title: "Default",
			dataIndex: "default",
			valueType: "switch",
		},
	]

	const handleGitGet = async () => {
		try {
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "GET",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return fetchGitApi
			} else {
				notification["info"]({
					message: "Empty Fetch",
					duration: 0,
				})
			}
		} catch (error) {
			console.log("Error handleGitGet ", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}
	const handleGitUpdate = async (contentStr: DataSourceType[]) => {
		try {
			const jsonStr = JSON.stringify(contentStr)
			const data = {
				message:
					"[skip ci] Update @maad-dev/gatsby-theme-backend/settings/languages.json from Modern backend.",
				content: btoa(jsonStr),
				sha: sha,
			}
			const fetchGitApi = await fetch(
				`https://api.github.com/repos/${process.env.GATSBY_GITHUB_OWNER}/${process.env.GATSBY_GITHUB_REPO}/contents/src/@maad-dev/gatsby-theme-backend/settings/languages.json`,
				{
					method: "PUT",
					headers: {
						authorization:
							"token " + process.env.GATSBY_GITHUB_PERSONAL_ACCESS_TOKEN,
						"Content-Type": "application/json; charset=UTF-8",
					},
					body: JSON.stringify(data),
				}
			)
				.then((response) => {
					return response.json()
				})
				.catch((error) => {
					console.log("Error fetchGitApi PUT ", error)
					notification["error"]({
						message: "Error code " + error.status,
						description: error.statusText,
						duration: 0,
					})
				})

			if (fetchGitApi) {
				return true
			}
		} catch (error) {
			console.log("Error handleGitUpdate!", error)
			notification["error"]({
				message: error.name + " code " + error.status,
				description: error.statusText,
				duration: 0,
			})
		}
		return false
	}

	useEffect(() => {
		if (!isAdministrator()) {
			notification["warning"]({
				message: "Warning!",
				description: "Your account is not allowed!",
			})
			navigate("/backend/dashboard")
		}

		const fetchDataSource = async () => {
			return await handleGitGet().then((getFile) => {
				if (getFile !== false) {
					const decodeContent = decodeUnicode(getFile.content)
					setDataSource(JSON.parse(decodeContent))
					setSha(getFile.sha)
				} else {
					setDataSource([])
				}
			})
		}
		fetchDataSource()
	}, [])

	return (
		<EditableProTable<DataSourceType>
			columns={columns}
			rowKey="key"
			value={dataSource}
			onChange={setDataSource}
			recordCreatorProps={false}
			toolBarRender={() => {
				return [
					<Button
						type="primary"
						key="save"
						disabled={!dataSource || dataSource.length === 0}
						onClick={async () => {
							const singleDefault = dataSource.filter((item) => {
								return item.default === true
							}).length
							if (singleDefault !== 1) {
								notification["warning"]({
									message: "Invalid!",
									description: "Languages must have only one default selected!",
									duration: 0,
								})
								return false
							}

							const validLanguage = dataSource.filter((item) => {
								return item.enabled === true && item.default === true
							}).length
							if (validLanguage === 0) {
								notification["warning"]({
									message: "Invalid!",
									description:
										"Languages must have at least one enabled & default!",
									duration: 0,
								})
								return false
							}
							// Process
							handleGitUpdate(dataSource).then((updateFile: any) => {
								if (updateFile === true) {
									notification["success"]({
										message: "Success!",
										description: "Updated languages successfully.",
										duration: 0,
									})
								} else {
									let errMsg = "No status"
									if (updateFile === 404) {
										errMsg = "Status: 404 Not Found"
									} else if (updateFile === 409) {
										errMsg = "Status: 409 Conflict"
									} else if (updateFile === 422) {
										errMsg = "Status: 422 Unprocessable Entity"
									}
									notification["error"]({
										message: "Error!",
										description: errMsg,
										duration: 0,
									})
								}
							})
						}}
					>
						Save
					</Button>,
				]
			}}
			editable={{
				type: "multiple",
				editableKeys,
				onValuesChange: (record, recordList) => {
					setDataSource(recordList)
				},
				onChange: setEditableRowKeys,
			}}
		/>
	)
}
